"""
文件上传工具
"""
import json
import math
import os
import tempfile
from typing import Literal

import requests
import typing_extensions

import openapi_client
from openapi_client.api.fileupload_api import FileuploadApi
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.baidu_pan_tool.core.baidu_pan_decorator_tools import auto_file_upload_api
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import PreCreateResult, SplitUploadResult, UploadCreateResult, \
    SplitUploadTempInfo
from afeng_tools.file_tool import file_tools
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


def _get_upload_config_file(file_id: str) -> str:
    """获取上传配置文件"""
    tmp_path = tempfile.gettempdir()
    save_path = os.path.join(tmp_path, 'BaiduPan')
    os.makedirs(save_path, exist_ok=True)
    return os.path.join(save_path, f'{file_id}.upload')


def _read_upload_config_range(file_id: str) -> SplitUploadTempInfo:
    """读取已经上传的分片范围列表（当断点续传时），"""
    downloaded_file = _get_upload_config_file(file_id)
    logger.info(f'[BiaduPan]文件分片上传，分片信息保存在[{downloaded_file}]')
    if os.path.exists(downloaded_file):
        return SplitUploadTempInfo.model_validate_json(file_tools.read_file(downloaded_file))


def _save_upload_config_range(file_id: str, temp_upload_info: SplitUploadTempInfo):
    """保存已经上传的分片信息"""
    file_tools.save_file(temp_upload_info.model_dump_json(), _get_upload_config_file(file_id))


def _delete_upload_config_range_file(file_id: str):
    """删除上传进度存储文件"""
    os.remove(_get_upload_config_file(file_id))


@auto_file_upload_api
def _upload_pre_create(access_token: str, pan_path: str, size: int, is_dir: bool,
                       block_list: list[str], rtype: Literal[1, 2, 3] | int = 2,
                       api_instance: FileuploadApi = None) -> PreCreateResult:
    """
    预上传：预上传是通知网盘云端新建一个上传任务，网盘云端返回唯一ID uploadid 来标识此上传任务。
    :param access_token: Access Token
    :param pan_path: 传后使用的文件绝对路径，如：/apps/appName/filename.jpg
            对于一般的第三方软件应用，路径以 "/apps/your-app-name/" 开头。对于小度等硬件应用，路径一般 "/来自：小度设备/" 开头。对于定制化配置的硬件应用，根据配置情况进行填写。
    :param size: 文件和目录两种情况：上传文件时，表示文件的大小，单位B；上传目录时，表示目录的大小，目录的话大小默认为0
    :param is_dir: 是否为目录，0 文件，1 目录
    :param block_list: 文件各分片MD5数组的json串。
                    block_list的含义如下:
                        如果上传的文件小于4MB，其md5值（32位小写）即为block_list字符串数组的唯一元素；
                        如果上传的文件大于4MB，需要将上传的文件按照4MB大小在本地切分成分片，不足4MB的分片自动成为最后一个分片，所有分片的md5值（32位小写）组成的字符串数组即为block_list。
    :param rtype: 文件命名策略。
                    1 表示当path冲突时，进行重命名
                    2 表示当path冲突且block_list不同时，进行重命名
                    3 当云端存在同名文件时，对该文件进行覆盖
    :param api_instance: 自动注入 FileuploadApi
    :return: PreCreateResult
    """
    try:
        block_list_str = json.dumps(block_list)
        api_response = api_instance.xpanfileprecreate(access_token, path=pan_path, isdir=1 if is_dir else 0,
                                                      size=size, block_list=block_list_str,
                                                      autoinit=1, rtype=rtype)
        logger.info(f"[BaiduPan]upload_pre_create[{pan_path}]:[{api_response}]")
        return PreCreateResult(**api_response)
    except openapi_client.ApiException as e:
        log_error(logger, f"[BaiduPan]upload_pre_create[{pan_path}]出现异常", e)


def _get_part_info_list(file_size: int) -> list[tuple[int, int]]:
    """
    获取分页信息列表
    :param file_size:
    :return: [(开始位置,读取长度)]
    """
    block_size = 4 * 1024 * 1024
    # 如果上传的文件大于4MB，需要将上传的文件按照4MB大小在本地切分成分片，不足4MB的分片自动成为最后一个分片，所有分片的md5值（32位小写）组成的字符串数组即为block_list。
    part_info_list = []
    part_num = math.ceil(file_size / block_size)
    for i in range(part_num):
        start_index = i * block_size
        part_info_list.append((start_index, block_size if i < part_num - 1 else file_size - start_index))
    return part_info_list


def _get_block_list(local_file: str, part_info_list: list[tuple[int, int]]) -> list[str]:
    """获取文件的block_list"""
    block_list = []
    with open(local_file, 'rb') as tmp_file:
        for tmp_start, tmp_length in part_info_list:
            tmp_file.seek(tmp_start)
            part_body = tmp_file.read(tmp_length)
            block_list.append(hashlib_tools.calc_byte_md5(part_body))
    return block_list


@auto_file_upload_api
def _split_upload(access_token: str, pan_path: str, upload_id: str, part_seq: int, body: bytes,
                  api_instance: FileuploadApi = None) -> SplitUploadResult:
    """
    分片上传：本接口用于将本地文件上传到网盘云端服务器。
    :param access_token: Accesss Token
    :param pan_path: 上传后使用的文件绝对路径
    :param upload_id: 上一个阶段预上传precreate接口下发的uploadid
    :param part_seq: 文件分片的位置序号，从0开始，参考上一个阶段预上传precreate接口返回的block_list
    :param body: 上传的文件内容
    :param api_instance:
    :return:
    """
    temp_file = os.path.join(tempfile.gettempdir(), f'{upload_id}.tmp')
    try:
        file_tools.save_file(body, temp_file, binary_flag=True)
        api_response = api_instance.pcssuperfile2(access_token, partseq=str(part_seq), path=pan_path,
                                                  uploadid=upload_id, type='tmpfile',
                                                  file=open(temp_file, 'rb'))
        logger.info(f"[BaiduPan]split_upload[{pan_path}]-[{part_seq}]:[{api_response}]")
        return SplitUploadResult(**api_response)
    except openapi_client.ApiException as e:
        log_error(logger, f"[BaiduPan]split_upload[{pan_path}]-[{part_seq}]出现异常", e)
    finally:
        if os.path.exists(temp_file):
            os.remove(temp_file)


@auto_file_upload_api
def _upload_create(access_token: str, pan_path: str, is_dir: bool, size: int,
                   upload_id: str, block_list: list[str], rtype: Literal[1, 2, 3] | int = 2,
                   api_instance: FileuploadApi = None) -> UploadCreateResult:
    """
    分片上传的第三步：创建文件，本接口用于将多个文件分片合并成一个文件，生成文件基本信息，完成文件的上传最后一步。
    :param access_token: Access Token
    :param pan_path: 上传后使用的文件绝对路径，需要与预上传precreate接口中的path保持一致
    :param is_dir: 是否目录，0 文件、1 目录，需要与预上传precreate接口中的isdir保持一致
    :param size: 文件或目录的大小，必须要和文件真实大小保持一致，需要与预上传precreate接口中的size保持一致
    :param upload_id: 预上传precreate接口下发的uploadid
    :param block_list: 文件各分片md5数组的json串
    :param rtype: 文件命名策略，默认1
                    0 为不重命名，返回冲突
                    1 为只要path冲突即重命名
                    2 为path冲突且block_list不同才重命名
                    3 为覆盖，需要与预上传precreate接口中的rtype保持一致
    :param api_instance: 自动注入FileuploadApi
    :return: UploadCreateResult
    """
    try:
        block_list_str = json.dumps(block_list)
        api_response = api_instance.xpanfilecreate(access_token, path=pan_path, isdir=1 if is_dir else 0,
                                                   size=size, uploadid=upload_id,
                                                   block_list=block_list_str, rtype=rtype)
        logger.info(f"[BaiduPan]upload_create[{pan_path}]:[{api_response}]")
        if api_response['errno'] == 0:
            return UploadCreateResult(**api_response)
        elif api_response['errno'] == -7:
            log_error(logger, f"[BaiduPan]文件或目录名错误或无权访问, upload_path:{pan_path}\n{api_response}")
        elif api_response['errno'] == -8:
            log_error(logger, f"[BaiduPan]文件或目录已存在, upload_path:{pan_path}\n{api_response}")
        elif api_response['errno'] == -10:
            log_error(logger, f"[BaiduPan]云端容量已满, upload_path:{pan_path}\n{api_response}")
        elif api_response['errno'] == 10:
            log_error(logger, f"[BaiduPan]创建文件失败, upload_path:{pan_path}\n{api_response}")
        elif api_response['errno'] == 31190:
            log_error(logger, f"[BaiduPan]文件不存在, upload_path:{pan_path}\n{api_response}")
        elif api_response['errno'] == 31365:
            log_error(logger, f"[BaiduPan]文件总大小超限, upload_path:{pan_path}\n{api_response}")
        else:
            log_error(logger, f"[BaiduPan]创建文件失败, upload_path:{pan_path}\n{api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, f"[BaiduPan]创建文件失败出现异常, upload_path:{pan_path}", e)


def upload_file(access_token: str, local_file: str, pan_path: str,
                rtype: Literal[1, 2, 3] | int = 2) -> UploadCreateResult | None:
    """
    上传文件到百度网盘
    :param access_token: Access Token
    :param local_file: 本地文件
    :param pan_path: 网盘路径
    :param rtype: 文件命名策略，默认1
                    0 为不重命名，返回冲突
                    1 为只要path冲突即重命名
                    2 为path冲突且block_list不同才重命名
                    3 为覆盖，需要与预上传precreate接口中的rtype保持一致
    :return: UploadCreateResult
    """
    file_id = hashlib_tools.calc_md5(f'{local_file}|{pan_path}')
    # 读取上传临时文件
    upload_temp_info = _read_upload_config_range(file_id)
    if not upload_temp_info:
        # 1、预上传
        file_size = 0 if os.path.isdir(local_file) else os.path.getsize(local_file)
        is_dir = os.path.isdir(local_file)
        part_info_list = _get_part_info_list(file_size)
        block_list = _get_block_list(local_file, part_info_list)
        pre_create_result = _upload_pre_create(access_token, pan_path, size=file_size, is_dir=is_dir,
                                               block_list=block_list, rtype=rtype)
        if pre_create_result.errno != 0:
            return
        upload_temp_info = SplitUploadTempInfo(file_size=file_size, is_dir=is_dir,
                                               part_info_list=part_info_list, upload_id=pre_create_result.uploadid,
                                               block_list=block_list)
        _save_upload_config_range(file_id, upload_temp_info)
    # 2、分片上传
    for index, (tmp_start, tmp_length) in enumerate(upload_temp_info.part_info_list):
        if upload_temp_info.uploaded_part_list and (tmp_start, tmp_length) in upload_temp_info.uploaded_part_list:
            continue
        split_upload_result = _split_upload(access_token, pan_path,
                                            upload_id=upload_temp_info.upload_id, part_seq=index,
                                            body=file_tools.read_file_body(local_file,
                                                                           start=tmp_start, length=tmp_length))
        if split_upload_result.md5 is None:
            return
        upload_temp_info.uploaded_part_list.append((tmp_start, tmp_length))
        _save_upload_config_range(file_id, upload_temp_info)
    # 3、创建文件
    upload_create_result = _upload_create(access_token, pan_path, is_dir=upload_temp_info.is_dir,
                                          size=upload_temp_info.file_size, upload_id=upload_temp_info.upload_id,
                                          block_list=upload_temp_info.block_list, rtype=rtype)
    # 删除上传临时存储文件
    _delete_upload_config_range_file(file_id)
    if upload_create_result.errno != 0:
        return
    return upload_create_result


@typing_extensions.deprecated('请使用upload_file(), 当前接口测试上传时没有成功，'
                              '出现{"error_code":31064,"error_msg":"file is not authorized"}')
def post_upload_file(access_token: str, local_file: str, pan_path: str,
                     on_dup: Literal['fail', 'overwrite', 'newcopy'] | str = 'newcopy') -> UploadCreateResult:
    """
    单步上传：本接口用于实现小文件单步上传一次HTTP请求交互即可完成上传的场景。
    :param access_token: Access Token
    :param local_file: 需要上传的本地文件
    :param pan_path: 上传的文件绝对路径
    :param on_dup: 上传的文件绝对路径冲突时的策略。fail（默认：冲突时失败）overwrite（冲突时覆盖） newcopy（冲突时重命名
    :return:
    """
    params = {
        'method': 'upload',
        'access_token': access_token,
        'path': http_params_tools.url_encode(pan_path),
        'ondup': on_dup
    }
    files = [
        ('file', open(local_file, 'rb'))
    ]
    headers = {'Host': 'pan.baidu.com'}
    post_url = 'https://d.pcs.baidu.com/rest/2.0/pcs/file?' + http_params_tools.url_encode_params(params)
    # print(post_url)
    try:
        response = requests.post(post_url, headers=headers, files=files)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            logger.info(f"[BaiduPan]post_upload_file[{local_file}]-[{pan_path}]:[{api_response}]")
            return UploadCreateResult(**api_response)
        else:
            log_error(logger,
                      f'[BaiduPan][{local_file}]-[{pan_path}]单步上传失败: {response.status_code}-{response.text}')
    except Exception as e:
        log_error(logger, f'[BaiduPan][{local_file}]-[{pan_path}]单步上传失败', e)



"""
泛型工具
"""
import sys

py_version = sys.version_info
if py_version >= (3, 8):
    from typing import get_args, TypeVar


def get_generic_type_arg(cls, arg_index: int = 0):
    """获取泛型值"""
    t = cls.__orig_bases__[0]
    if py_version >= (3, 8):
        return get_args(t)[arg_index]
    else:
        return t.__args__[arg_index]


def get_generic_type(cls, arg_index: int = 0):
    """获取泛型类型"""
    generic_type = get_generic_type_arg(cls, arg_index)
    if isinstance(generic_type, TypeVar):
        return cls.model_type
    return generic_type



from abc import abstractmethod
from typing import Callable
from starlette.requests import Request
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.common.service.error_base_service import DefaultErrorService
from afeng_tools.fastapi_tool.template import template_render_service, template_service
from afeng_tools.fastapi_tool.template.item import TemplateBreadCrumbData, Error404DataItem, Error500DataItem, \
    Error501DataItem
from afeng_tools.http_tool import http_request_tools


class DefaultAppErrorService(DefaultErrorService):
    @staticmethod
    @abstractmethod
    def get_template_page_header_func() -> Callable[[...], str]:
        """获取模板页头的函数"""
        pass

    @staticmethod
    @abstractmethod
    def get_template_page_footer_func() -> Callable[[...], str]:
        """获取模板页底部的函数"""
        pass

    @staticmethod
    @abstractmethod
    def get_app_info() -> AppInfo:
        pass

    def get_common_template_context_data(self, is_mobile: bool, error_code: int, error_title: str):
        group_code = f'error/{error_code}'
        template_tuple_list = [
            ('template_head', template_render_service.get_template_head(
                group_code=group_code,
                context_data_func=lambda: template_service.create_html_head_data(_app_info=self.get_app_info(),
                                                                                 is_mobile=is_mobile,
                                                                                 title=error_title),
                reload_freq=ReloadFreqEnum.weekly
            )),
            ('template_bread_crumb', template_render_service.get_template_bread_crumb(
                group_code=group_code,
                context_data_func=lambda: TemplateBreadCrumbData(is_mobile=is_mobile,
                                                                 page_title=error_title,
                                                                 bread_crumb_list=[]),
                reload_freq=ReloadFreqEnum.yearly
            )),
        ]
        template_page_header_func = self.get_template_page_header_func()
        if template_page_header_func:
            template_tuple_list.append(('template_page_header', template_page_header_func(is_mobile=is_mobile)))
        template_page_footer_func = self.get_template_page_footer_func()
        if template_page_footer_func:
            template_tuple_list.append(('template_page_footer', template_page_footer_func(is_mobile=is_mobile)))
        context_data = {
            'is_mobile': is_mobile,
            'title': error_title,
        }
        for template_name, template_file in template_tuple_list:
            context_data[template_name] = template_file
        return context_data

    def handle_404(self, error_data: Error404DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        tmp_app_info = self.get_app_info()
        if not tmp_app_info:
            app_info = tmp_app_info
        common_context_data = self.get_common_template_context_data(
            is_mobile=http_request_tools.is_mobile(request.headers.get('user-agent')),
            error_code=404,
            error_title='404-Not Found')
        if context_data:
            common_context_data.update(context_data)
        return super().handle_404(error_data, request=request, context_data=common_context_data,
                                  app_info=app_info)

    def handle_500(self, error_data: Error500DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        tmp_app_info = self.get_app_info()
        if not tmp_app_info:
            app_info = tmp_app_info
        common_context_data = self.get_common_template_context_data(
            is_mobile=http_request_tools.is_mobile(request.headers.get('user-agent')),
            error_code=500,
            error_title='500-服务器错误')
        if context_data:
            common_context_data.update(context_data)
        if not error_data.contact_info_url:
            error_data.contact_info_url = f'mailto:{app_info.contact_email}'
        return super().handle_500(error_data, request=request, context_data=common_context_data,
                                  app_info=app_info)

    def handle_501(self, error_data: Error501DataItem, request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        tmp_app_info = self.get_app_info()
        if not tmp_app_info:
            app_info = tmp_app_info
        common_context_data = self.get_common_template_context_data(
            is_mobile=http_request_tools.is_mobile(request.headers.get('user-agent')),
            error_code=501,
            error_title='501-操作失败')
        if context_data:
            common_context_data.update(context_data)
        if not error_data.contact_info_url:
            error_data.contact_info_url = f'mailto:{app_info.contact_email}'
        if not error_data.feedback_url:
            error_data.feedback_url = app_info.feedback_url
        return super().handle_501(error_data, request=request, context_data=common_context_data,
                                  app_info=app_info)



from typing import Callable, Optional, Any

from pydantic import BaseModel
from starlette.requests import Request


class FastapiConfigItem(BaseModel):
    is_json_api: Optional[bool] = None
    # 错误404创建context_data的函数，参数：(message: str, is_mobile: bool = False)
    error404_context_data_func: Optional[Callable[[str, bool], dict]] = None
    # 错误500创建context_data的函数，参数：(message: str, is_mobile: bool = False)
    error500_context_data_func: Optional[Callable[[str, bool], dict]] = None
    # 500异常后后台工作函数，如发送邮件, async def send_email(request: Request, exception: Exception, traceback_msg: str)
    error500_background_work_func: Optional[Callable[[Request, Exception, str], None]] = None
    # 错误501创建context_data的函数，参数：(message: str, is_mobile: bool = False)
    error501_context_data_func: Optional[Callable[[str, bool], dict]] = None


class ResponseResult(BaseModel):
    """响应结果"""
    # 错误码
    error_no: int = 0
    # 提示信息
    message: Optional[str] = 'success'
    # 子消息
    sub_message: Optional[str] = None
    # 响应数据
    data: Optional[Any] = None


class ResponseTemplateResult(BaseModel):
    # 模板文件，如： f'{app_info.db_code}' + '/views/error/404.html'
    template_file: str
    # 标题
    title: str
    # 消息
    message: str
    # 上下文数据
    context_data: Optional[dict[str, Any]] = None



"""
内存缓存工具
"""
import time
from typing import Any

MEMORY_CACHE = {}


def add_cache(group_code: str, key: str, value: Any):
    """添加缓存"""
    group_cache = MEMORY_CACHE.get(group_code)
    if not group_cache:
        MEMORY_CACHE[group_code] = dict()
    MEMORY_CACHE[group_code][key] = value


def delete_cache(group_code: str, key: str) -> Any:
    """添加缓存"""
    group_cache = MEMORY_CACHE.get(group_code)
    if group_cache:
        return group_cache.pop(key, None)


def get_cache(group_code: str, key: str) -> Any:
    """获取缓存"""
    group_cache = MEMORY_CACHE.get(group_code)
    if group_cache:
        return group_cache.get(key)


def add_time_cache(group_code: str, key: str, value: Any, timestamp: float = None):
    """添加缓存"""
    group_cache = MEMORY_CACHE.get(group_code)
    if not group_cache:
        MEMORY_CACHE[group_code] = dict()
    if timestamp is None:
        timestamp = time.time()
    MEMORY_CACHE[group_code][key] = (timestamp, value)


def get_time_cache(group_code: str, key: str) -> tuple[float, Any]:
    """获取缓存: 时间戳，值"""
    group_cache = MEMORY_CACHE.get(group_code)
    if group_cache:
        return group_cache.get(key)



from pandas import DataFrame

from afeng_tools.tushare_tool.tushare_tools import get_api


def get_stock_list() -> DataFrame:
    """沪深股票-股票列表"""
    ts_api = get_api()
    query_param = {"ts_code": "", "name": "", "exchange": "", "market": "", "is_hs": "", "list_status": "", "limit": "",
                   "offset": ""}
    query_field_list = ["ts_code", "stock_code", "name", "area", "industry", "market", "list_date",
                        "fullname", "enname", "cnspell", "exchange", "curr_type", "list_status",
                        "delist_date", "is_hs", "act_name", "act_ent_type"]
    # 拉取数据
    return ts_api.stock_basic(**query_param, fields=query_field_list)


def get_trade_calendar() -> DataFrame:
    """沪深股票-交易日历"""
    ts_api = get_api()
    query_param = {"exchange": "", "cal_date": "", "start_date": "", "end_date": "", "is_open": "", "limit": "",
                   "offset": ""}
    field_list = ["exchange", "cal_date", "is_open", "pretrade_date"]
    # 拉取数据
    return ts_api.trade_cal(**query_param, fields=field_list)



"""
- pip install pytest-playwright -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

import os.path

from playwright.sync_api import Response
from afeng_tools.file_tool import file_tools, tmp_file_tools

auth_qrcode_image = os.path.join(tmp_file_tools.get_user_tmp_dir(), 'baidu_auth_qrcode.png')


def filter_login_qrcode_api(response: Response):
    if 'passport.baidu.com/v2/api/qrcode' in response.url:
        file_tools.save_file(response.body(), auth_qrcode_image, binary_flag=True)


def delete_login_qrcode_image():
    """删除扫码登录图片"""
    if os.path.exists(auth_qrcode_image):
        os.remove(auth_qrcode_image)



from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class RelationPoService(PoService):
    """
    使用示例：relation_po_service = RelationPoService(app_info.db_code, RelationInfoPo)
    """
    _table_name_ = "tb_relation_info"
    pass



from typing import Any, TypeVar, Generic, Callable, Dict

from afeng_tools.id_code_tool import id_code_tools
from afeng_tools.sqlalchemy_tools import sqlalchemy_settings
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_enums import SortTypeEnum
from afeng_tools.sqlalchemy_tools.crdu import base_crdu
from sqlalchemy.orm import Query
from sqlalchemy.sql._typing import _ColumnExpressionOrStrLabelArgument

ModelType = TypeVar('ModelType', bound=Any)


class PoService(Generic[ModelType]):
    # 表名
    _table_name_: str = None

    def __init__(self, db_code: str, model_type=None):
        self.db_code = db_code
        self.sqlalchemy_db = sqlalchemy_settings.get_database(db_code)
        self.Base = self.sqlalchemy_db.Base
        if model_type is not None:
            self.model_type = model_type
        else:
            if hasattr(self, 'model_type') and self.model_type is None and self._table_name_:
                self.model_type = self._get_model_type()
            elif self._table_name_:
                self.model_type = self._get_model_type()

    def _get_model_type(self):
        if not self.sqlalchemy_db.mapper_dict:
            self.sqlalchemy_db.mapper_dict = {tmp_mapper.local_table.name: tmp_mapper.entity
                                              for tmp_mapper in self.Base.registry.mappers}
        model_type = self.sqlalchemy_db.mapper_dict.get(self._table_name_)
        if model_type is None:
            self.sqlalchemy_db.mapper_dict = {tmp_mapper.local_table.name: tmp_mapper.entity
                                              for tmp_mapper in self.Base.registry.mappers}
            model_type = self.sqlalchemy_db.mapper_dict.get(self._table_name_)
        return model_type

    def __class_getitem__(cls, item):
        if '__table__' in item.__dict__:
            cls.model_type = item
        return super().__class_getitem__(item)

    def add(self, po: ModelType) -> ModelType:
        return base_crdu.add(po, db_code=self.db_code)

    def add_batch(self, model_list: list[ModelType]) -> list[ModelType]:
        return base_crdu.add_all(model_list, db_code=self.db_code)

    def update(self, po: ModelType, *criterion) -> ModelType:
        return base_crdu.update(po, *criterion, db_code=self.db_code)

    def update_batch(self, model_list: list[ModelType]) -> list[ModelType]:
        return base_crdu.update_batch(model_list, db_code=self.db_code)

    def save(self, po: ModelType, *criterion, auto_code: bool = False, exist_update: bool = True) -> ModelType:
        po = base_crdu.save(po, *criterion, db_code=self.db_code, exist_update=exist_update)
        if auto_code:
            if po.code is None:
                po.code = id_code_tools.get_code_by_id(po.id)
                po = base_crdu.update(po, db_code=self.db_code)
        return po

    def save_batch(self, model_list: list[ModelType], criterion_fun: Callable,
                   exist_update: bool = True) -> list[ModelType]:
        """
        批量保存
        :param model_list:  Model数据列表
        :param criterion_fun: 通过接受model，返回判断条件, 如：lambda x: type(x).ts_code == x.ts_code
        :param exist_update: 如果存在则更新，如果为False，则如果存在则啥也不做
        :return: Model数据列表
        """
        return base_crdu.save_batch_by_where(model_list, criterion_fun, exist_update=exist_update, db_code=self.db_code)

    def delete(self, *criterion):
        return base_crdu.delete(self.model_type, *criterion, db_code=self.db_code)

    def create_query(self, *criterion) -> Query:
        return base_crdu.create_query(self.model_type, *criterion, db_code=self.db_code)

    def get(self, *criterion) -> ModelType:
        return base_crdu.query_one(self.model_type, *criterion, db_code=self.db_code)

    def query_more(self, *criterion, sort_column: str = 'order_num',
                   sort_type: SortTypeEnum = SortTypeEnum.desc) -> list[ModelType]:
        return base_crdu.query_all(self.model_type, *criterion, sort_column=sort_column, sort_type=sort_type,
                                   db_code=self.db_code)

    def query_page(self, page_num: int, page_size: int, *criterion,
                   sort_column: str = 'order_num',
                   sort_type: SortTypeEnum = SortTypeEnum.desc) -> list[ModelType]:
        return base_crdu.query_page(self.model_type, page_num, page_size, *criterion,
                                    sort_column=sort_column, sort_type=sort_type,
                                    db_code=self.db_code)

    def count(self, *criterion) -> int:
        return base_crdu.count(self.model_type, *criterion, db_code=self.db_code)

    def query_count(self, group_column: _ColumnExpressionOrStrLabelArgument[Any], *criterion) -> dict[str, int]:
        """分组查询数量， 如：count_dict = query_count(BookInfoPo.category_code)"""
        return base_crdu.query_count(group_column, *criterion, db_code=self.db_code)



from afeng_tools.baidu_pan_tool import baidu_pan_tools
from afeng_tools.baidu_pan_tool.baidu_pan_enum import BaidupanConfigKeyEnum
from afeng_tools.baidu_pan_tool.baidu_pan_tools import get_access_token, list_file, get_share_url
from afeng_tools.baidu_pan_tool.tools import baidu_pan_user_tools, baidu_pan_file_path_tools
from afeng_tools.baidu_pan_tool.tools.baidu_pan_audio_file_tools import list_audio_category, list_audio_category, list_audio, \
    get_audio_m3u8_url
from afeng_tools.baidu_pan_tool.tools.baidu_pan_file_download_tools import download_file, get_download_url
from afeng_tools.baidu_pan_tool.tools.baidu_pan_file_upload_tools import upload_file, post_upload_file
from afeng_tools.baidu_pan_tool.tools.baidu_pan_share_tools import create_share, verify_share_pwd
from afeng_tools.baidu_pan_tool.tools.baidu_pan_video_file_tools import get_video_m3u8_url
from afeng_tools.file_tool import file_tools
from afeng_tools.baidu_pan_tool.baidu_pan_settings import get_config

def auth_demo():
    access_token = get_access_token()
    # print(user_tools.get_user_info(access_token))
    # print(user_tools.get_pan_info(access_token))
    result = list_file(access_token=access_token, dir_path='/disk/book/SEO')
    print(result.file_list)


def download_demo():
    fs_id = 36779198899466  # 428389752925140
    access_token = get_access_token()
    # download_file(access_token, fs_id, save_path='', max_size=5 * 1000 * 1000)
    down_url = get_download_url(access_token, fs_id)
    print(down_url)


def upload_demo():
    access_token = get_access_token()
    # tmp_file = r'C:\迅雷云盘\《无所畏惧：颠覆你内心的脆弱》\《无所畏惧：颠覆你内心的脆弱》.pdf'
    # upload_file(access_token, local_file=tmp_file, pan_path='pan_root_path+/tmp/test.pdf')
    # tmp_file = r'C:\迅雷云盘\《无所畏惧：颠覆你内心的脆弱》\elon.png'
    # result = upload_file(access_token, local_file=tmp_file, pan_path=pan_root_path + '/tmp/elon.png')
    tmp_file = r'C:\www\tmp\AutoToolSpace\ali_tmp_file\【公众号：zsxx_xxyg】16.电力系统一次电气设备简介(重点).ppt'
    result = upload_file(access_token, local_file=tmp_file,
                         pan_path=get_config(BaidupanConfigKeyEnum.pan_root_path) + '/learn/电气/电力系统/16.电力系统一次电气设备简介(重点).ppt')
    print(result)


def image_demo():
    pass


def audio_demo():
    access_token = get_access_token()
    result = list_audio_category(access_token=access_token)
    for tmp in result.data_list:
        list_audio_result = list_audio(access_token, mb_id=tmp.mb_id, show_meta=1)
        if list_audio_result.data_list:
            for tmp_audio in list_audio_result.data_list:
                m3u8_url, headers = get_audio_m3u8_url(access_token, tmp_audio.path)
                print(m3u8_url)


def video_demo():
    access_token = get_access_token()
    m3u8_url, headers = get_video_m3u8_url(access_token, pan_path='/电影/阿凡达.mp4')
    print(m3u8_url)


def share_demo():
    access_token = get_access_token()
    share_result = create_share(get_config(BaidupanConfigKeyEnum.app_id), access_token, fs_id_list=['36779198899466'], pwd='yyds')
    # print(share_result)
    verify_result = verify_share_pwd(get_config(BaidupanConfigKeyEnum.app_id), access_token, short_url='bqCUUYeH9HRJNxz6Y8U7nA', pwd='1234')
    print(verify_result)


def share_url_demo():
    # share_url = get_share_url('https://pan.baidu.com/s/1cmB00')
    # print(share_url)
    # share_url = get_share_url(
    #     '链接:https://pan.baidu.com/s/1C1sbTf65U-YwYCw06nHKFQ 提取码:gxlk hi，这是我用百度网盘分享的内容~复制这段内容打开「百度网盘」APP即可获取')
    # print(share_url)
    # share_url = get_share_url(
    #     '数学奥林匹克小丛书https://pan.baidu.com/share/link?uk=3443847511&shareid=82423329?pwd=yyds 提取码:gxlk hi，这')
    # print(share_url)
    line_list = file_tools.read_file_lines('share_msg.txt')
    for tmp_line in line_list:
        share_url = get_share_url(tmp_line)
        if share_url:
            print(share_url)


def baidu_pan_path_func(x):
    return '/book/暂定/' + x if x else '/book/暂定/2023年优质书籍（1-9月）'


def list_all_file_demo():
    access_token = baidu_pan_tools.get_access_token()
    exist_result = file_path_tools.list_all_by_page(access_token,
                                                    dir_path=get_config(BaidupanConfigKeyEnum.pan_root_path) + '/' + (baidu_pan_path_func('').strip('/')),
                                                    recursion=1)
    exist_file_name_list = []
    if exist_result:
        exist_file_name_list = [tmp.server_filename for tmp in exist_result.file_list]
    print(exist_file_name_list)

if __name__ == '__main__':
    # auth_demo()
    # download_demo()
    # upload_demo()
    # audio_demo()
    # video_demo()
    # share_demo()
    # share_url_demo()
    list_all_file_demo()



from typing import Callable

from starlette.requests import Request

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool import fastapi_router_tools
from afeng_tools.fastapi_tool.template import template_service
from afeng_tools.fastapi_tool.template.item.page_template_items import ArticleTemplateItem


class ArticleView:
    """
    使用示例：router = ArticleView(app_info.db_code, ResourceInfoPo).router
    """

    def __init__(self, app_info: AppInfo,
                 detail_context_func: Callable[[Request, str], ArticleTemplateItem] = None,
                 type_context_func: Callable[[Request, str, str], ArticleTemplateItem] = None,
                 detail_page_template: str = '{app_code}/views/detail/index.html',
                 type_page_template: str = '{app_code}/views/{type_code}/index.html'):
        """
        是否缓存页面
        :param app_info:
        :param detail_context_func: 详情页内容函数(request, article_code)
        :param type_context_func: 类型页内容函数(request, type_code, article_code)
        :param detail_page_template: 详情页模板, 默认是：   '{app_code}/views/detail/index.html'
        :param type_page_template: 类型页模板，默认是：'{app_code}/views/{type_code}/index.html'
        """
        self.app_info = app_info
        self.detail_context_func = detail_context_func
        self.type_context_func = type_context_func
        self.detail_page_template = detail_page_template
        self.type_page_template = type_page_template
        self._router = fastapi_router_tools.create_router(prefix='/article', tags=['文章'])
        if detail_context_func:
            self._router.get('/detail/{article_code}')(self.detail_page)
        if type_context_func:
            self._router.get('/{type_code}/{article_code}')(self.type_page)

    @property
    def router(self):
        return self._router

    async def detail_page(self, request: Request, article_code: str):
        return template_service.render_template(request,
                                                template_file=self.detail_page_template.format(
                                                    app_code=self.app_info.code),
                                                template_item=self.detail_context_func(request, article_code))

    async def type_page(self, request: Request, type_code: str, article_code: str):
        return template_service.render_template(request,
                                                template_file=self.type_page_template.format(
                                                    app_code=self.app_info.code,
                                                    type_code=type_code),
                                                template_item=self.type_context_func(request, type_code, article_code))



from afeng_tools.pydantic_tool.model.common_models import LinkItem


def get_pagination_data(page_num: int, page_size: int, total_page: int, href_prefix: str,
                        up_num: int = 3,
                        page_num_attr: str = 'page_num',
                        page_size_attr: str = 'page_size') -> list[LinkItem]:
    """获取分页值"""
    start_num = page_num - 4
    if start_num <= 0:
        start_num = 1
    end_num = page_num + up_num
    if end_num > total_page:
        end_num = total_page
    result_list = []
    for tmp_num in range(start_num, end_num + 1):
        result_list.append(LinkItem(title=str(tmp_num), is_active=(page_num == tmp_num),
                                    href=f'{href_prefix}&{page_num_attr}={tmp_num}&{page_size_attr}={page_size}'))
    return result_list


def init_page_value(page_num: int, page_size: int, default_page_size: int = 10,
                    max_page_size: int = 50) -> tuple[int, int]:
    """初始化分页值"""
    if page_num <= 0:
        page_num = 1
    if page_size <= 0 or page_size > max_page_size:
        page_size = default_page_size
    return page_num, page_size


def create_page_btn(page_num: int, page_size: int, total_page: int, href_prefix: str,
                    pre_btn_title: str = '上一页', next_btn_title: str = '下一页',
                    page_num_attr: str = 'page_num',
                    page_size_attr: str = 'page_size') -> tuple[int, LinkItem, LinkItem]:
    """创建分页按钮"""
    if page_num > total_page:
        page_num = total_page
    pre_btn, next_btn = None, None
    if page_num > 1:
        pre_btn = LinkItem(title=pre_btn_title,
                           href=f'{href_prefix}&{page_num_attr}={page_num - 1}&{page_size_attr}={page_size}')
    if page_num < total_page:
        next_btn = LinkItem(title=next_btn_title,
                            href=f'{href_prefix}&{page_num_attr}={page_num + 1}&{page_size_attr}={page_size}')
    return page_num, pre_btn, next_btn



"""
模板装饰器工具
"""
from typing import Callable

from afeng_tools.decorator_tool import decorator_tools


def html_template(name: str, file: str):
    """装饰器：html模板"""

    def func_wrapper(func: Callable):
        def inner_wrapper(*args, **kwargs):
            if 'template_name' not in kwargs:
                kwargs['template_name'] = name
            if 'template_file' not in kwargs:
                kwargs['template_file'] = file
            return decorator_tools.run_func(func, *args, **kwargs)

        return inner_wrapper

    return func_wrapper


def template_area(file: str):
    """装饰器：html模板区域"""

    def func_wrapper(func: Callable):
        def inner_wrapper(*args, **kwargs):
            if 'template_file' not in kwargs:
                kwargs['template_file'] = file
            return decorator_tools.run_func(func, *args, **kwargs)

        return inner_wrapper

    return func_wrapper



"""
获取文件元信息工具
"""
import requests

import openapi_client
from afeng_tools.baidu_pan_tool.tools import baidu_pan_file_path_tools
from openapi_client.api.multimediafile_api import MultimediafileApi
from afeng_tools.baidu_pan_tool.core.baidu_pan_decorator_tools import auto_media_file_api
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import FileMetaInfo, CategoryCountInfo, SearchFileInfo
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


def get_file_meta_by_path(access_token: str, file_pan_path: str) -> SearchFileInfo:
    """
       通过文件路径查询文件信息: 本接口可用于获取用户指定文件的meta信息。  meta信息包括文件名字、文件创建时间、文件的下载地址等
       :param access_token: Access Token
       :param file_pan_path: 网盘文件路径，如： %2Fapps%2F转存%2F偏方秘方%2F食疗是最好的偏方.pdf
       :return:
       """
    if file_pan_path.startswith('%2F'):
        file_pan_path = http_params_tools.url_decode(file_pan_path)
    file_info_arr = file_pan_path.rsplit('/', maxsplit=1)
    search_result = baidu_pan_file_path_tools.search(access_token, key_word=file_info_arr[1], dir_path=file_info_arr[0])
    if search_result and search_result.file_list:
        return search_result.file_list[0]


def get_file_meta(access_token: str, fs_id: int, d_link: int = 0,
                  thumb: int = 0, extra: int = 0, need_media: int = 0, ) -> FileMetaInfo:
    """
       查询文件信息: 本接口可用于获取用户指定文件的meta信息。 meta信息包括文件名字、文件创建时间、文件的下载地址等。
       :param access_token: Access Token
       :param fs_id: 文件id
       :param d_link: 是否需要下载地址，0为否，1为是，默认为0。
       :param thumb: 是否需要缩略图地址，0为否，1为是，默认为0
       :param extra: 图片是否需要拍摄时间、原图分辨率等其他信息，0 否、1 是，默认0
       :param need_media: 视频是否需要展示时长信息，needmedia=1时，返回 duration 信息时间单位为秒 （s），转换为向上取整。 0 否、1 是，默认0
       :return:
       """
    file_list = get_file_metas(access_token, fs_id_list=[fs_id], d_link=d_link,
                               thumb=thumb, extra=extra, need_media=need_media)
    if file_list:
        return file_list[0]


@auto_media_file_api
def get_file_metas(access_token: str, fs_id_list: list[int], d_link: int = 0,
                   thumb: int = 0, extra: int = 0, need_media: int = 0,
                   api_instance: MultimediafileApi = None) -> list[FileMetaInfo]:
    """
    查询文件信息: 本接口可用于获取用户指定文件的meta信息。支持查询多个或一个文件的meta信息，meta信息包括文件名字、文件创建时间、文件的下载地址等。
    :param access_token: Access Token
    :param fs_id_list: 文件id数组，数组中元素是uint64类型，数组大小上限是：100
    :param d_link: 是否需要下载地址，0为否，1为是，默认为0。
    :param thumb: 是否需要缩略图地址，0为否，1为是，默认为0
    :param extra: 图片是否需要拍摄时间、原图分辨率等其他信息，0 否、1 是，默认0
    :param need_media: 视频是否需要展示时长信息，needmedia=1时，返回 duration 信息时间单位为秒 （s），转换为向上取整。 0 否、1 是，默认0

    :param api_instance:
    :return:
    """
    try:
        api_response = api_instance.xpanmultimediafilemetas(access_token,
                                                            fsids=f"[{','.join([str(tmp) for tmp in fs_id_list])}]",
                                                            thumb=str(thumb), extra=str(extra), dlink=str(d_link),
                                                            needmedia=need_media)
        # print(api_response)
        if api_response['errno'] == 0:
            return [FileMetaInfo(**tm_data) for tm_data in api_response['list']]
        else:
            log_error(logger, f"[BaiduPan]查询文件{fs_id_list}元信息失败，api_response: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling MultimediafileApi->xpanmultimediafilemetas", e)


def list_category_info(access_token: str, dir_path: str = '/',
                       category: int = None, recursion: int = 0) -> dict[int, CategoryCountInfo]:
    """
    获取分类文件总个数: 本接口用于获取用户指定目录下指定类型的文件数量。
    :param access_token: Access Token
    :param dir_path: 目录名称，以/开头的绝对路径, 默认为/
    :param category: 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    :param recursion: 是否需要递归，0为不需要，1为需要，默认为0
                        递归是指：当目录下有文件夹，使用此参数，可以获取到文件夹下面的bt文件
    :return: {category:CategoryCountInfo}
    """
    params = {
        'method': 'btlist',
        'access_token': access_token,
        'parent_path': http_params_tools.url_encode(dir_path),
        'category': category,
        'recursion': recursion,
    }
    headers = {
        'User-Agent': 'pan.baidu.com'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/file', params=params, headers=headers)
        response.encoding = 'utf8'
        api_response = response.json()
        if api_response['errno'] == 0:
            return {int(tmp_key): CategoryCountInfo(**api_response['info'].get(tmp_key)) for tmp_key in
                    api_response['info'].keys()}
        else:
            log_error(logger, f"[BaiduPan][{dir_path}]下获取分类文件总个数: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling MultimediafileApi->xpanlistcategory", e)
